import $axios from 'axios'
import { colors } from 'quasar'
const { hexToRgb } = colors
import { toPng, toJpeg, toSvg } from 'html-to-image'

let availableKeys = null
const cachedMaps = []

export const zhutils = {
  dateToday () {
    return new Date().toISOString().split('T')[0]
  },
  dateTime () {
    const dt = new Date().toISOString().split('T')
    return dt[0] + '_' + dt[1].substr(0, 5).replace(/:/g, '-')
  },
  sortToReadableDate (sortDate) {
    return sortDate.split('-').reverse().join('.')
  },
  getRandomInt (min, max) {
    min = Math.ceil(min)
    max = Math.floor(max)
    return Math.floor(Math.random() * (max - min + 1)) + min
  },
  pointAddition (p1, p2) {
    return { x: p1.x + p2.x, y: p1.y + p2.y }
  },
  pointOverlapRect (point, rect) {
    return zhutils.rectOverlapRect({ l: point.x, t: point.y, r: point.x + 0.00001, b: point.y + 0.00001 }, rect)
  },
  rectOverlapRect (r1, r2) {
    return Math.max(r1.l, r2.l) < Math.min(r1.r, r2.r) && Math.max(r1.t, r2.t) < Math.min(r1.b, r2.b)
  },
  rectCenter (rect) {
    return { x: (rect.l + rect.r) / 2, y: (rect.t + rect.b) / 2 }
  },
  rectSize (rect) {
    return { w: rect.r - rect.l, h: rect.b - rect.t }
  },
  rectInRectBounds (r1, r2) {
    const size = zhutils.rectSize(r1)
    return {
      l: Math.max(Math.min(r1.l, r2.r - size.w), r2.l),
      t: Math.max(Math.min(r1.t, r2.b - size.h), r2.t),
      r: Math.max(Math.min(r1.r, r2.r), r2.l + size.w),
      b: Math.max(Math.min(r1.b, r2.b), r2.t + size.h)
    }
  },
  combineRects (r1, r2) {
    return { l: r1.l + r2.l, t: r1.t + r2.t, r: r1.r + r2.r, b: r1.b + r2.b }
  },
  copyObj(obj) {
    return JSON.parse(JSON.stringify(obj))
  },
  isNumeric (numberString) {
    return !isNaN(parseFloat(numberString)) && isFinite(numberString)
  },
  formatNumber (number) {
    if (this.isNumeric(number)) {
      return number.toLocaleString('de-ch')
    }
    return number
  },
  formatChTel (string) {
    return string.replace(/.*([0-9]{2})\W*([0-9]{3})\W*([0-9]{2})\W*([0-9]{2})$/, '+41 $1 $2 $3 $4')
  },
  valueBetween (start, end, amt) { // 0 - 1
    if (start === end) {
      return end
    }
    return parseFloat(((amt - start) * (1 / (end - start))).toFixed(3))
  },
  lerp (start, end, amt) {
    return (1 - amt) * start + amt * end
  },
  lerpColor (a, b, amount) { 
    const ah = parseInt(a.replace(/#/g, ''), 16),
      ar = ah >> 16, ag = ah >> 8 & 0xff, ab = ah & 0xff,
      bh = parseInt(b.replace(/#/g, ''), 16),
      br = bh >> 16, bg = bh >> 8 & 0xff, bb = bh & 0xff,
      rr = ar + amount * (br - ar),
      rg = ag + amount * (bg - ag),
      rb = ab + amount * (bb - ab)
    return '#' + ((1 << 24) + (rr << 16) + (rg << 8) + rb | 0).toString(16).slice(1)
  },
  contrastColor (hexString) {
    const rgb = hexToRgb(hexString)
    return ((rgb.r * 0.299) + (rgb.g * 0.587) + (rgb.b * 0.114)) > 130 ? '#000' : '#fff'
  },
  hexToRgb (hex) {
    const rgb = hex.replace(/^#?([a-f\d])([a-f\d])([a-f\d])$/i ,(m, r, g, b) => '#' + r + r + g + g + b + b)
      .substring(1).match(/.{2}/g)
      .map(x => parseInt(x, 16))
    return { r: rgb[0], g: rgb[1], b: rgb[2] }
  },
  rgbTohex({ r, g, b }) {
    r = Math.round(r)
    g = Math.round(g)
    b = Math.round(b)
    r = Math.min(r, 255)
    g = Math.min(g, 255)
    b = Math.min(b, 255)
    return '#' + [r, g, b].map(c => c.toString(16).padStart(2, '0')).join('')
  },
  colorOverlayOnWhite (hex, a) {
    const rgb = hexToRgb(hex)
    rgb.r = Math.floor(rgb.r * a + 0xff * (1 - a))
    rgb.g = Math.floor(rgb.g * a + 0xff * (1 - a))
    rgb.b = Math.floor(rgb.b * a + 0xff * (1 - a))
    return zhutils.rgbTohex(rgb)
  },
  arrayIntersectByKey (list1, list2, key = 'id') {
    const newArray = []
    list1.forEach(item1 => {
      const item2 = list2.find(o => o[key] === item1[key])
      if (item2) {
        newArray.push({ ...item2, ...item1 })
      }
    })
    return newArray
  },
  arrayGroupByKey (array, key) {
    return Object.entries(array.reduce((hash, obj) => {
      if(obj[key] === undefined) return hash; 
      return Object.assign(hash, { [obj[key]]:( hash[obj[key]] || [] ).concat(obj)})
    }, {})).map(o => o[1])
  },
  arraySum (array) {
    return array.reduce((a, b) => a + b, 0)
  },
  animate (start, end, duration, callback, competed) {
    const props = {
      start: start,
      end: end,
      duration: duration,
      callback: callback,
      competed: competed,
      endTime: new Date().getTime() + duration // durtion in milisec
    }
    this.animateLoop(props)
  },
  animateLoop (props, now = new Date().getTime()) {
    const frames = 50
    const miliseconds = 1000 / frames
    if (now < props.endTime) {
      setTimeout(() => {
        const value = (props.duration - (props.endTime - now)) / props.duration
        props.callback(this.lerp(props.start, props.end, value), value)
        this.animateLoop(props)
      }, miliseconds)
    } else {
      props.callback(props.end, 1.0)
      if (props.competed) {
        props.competed()
      }
    }
  },
  async getMapKeys (path = 'https://www.web.statistik.zh.ch/zhwebcomponent/map/') {
    return await new Promise(function(resolve) {
      if(availableKeys) {
        resolve(availableKeys)
      } else {
        $axios.get(path + 'summary.json').then(response => {
          availableKeys = response.data.groups.filter(o => o.filetype === 'json')
          resolve(availableKeys)
        })
      }
    })
  },
  async getMapDataWithKey (key, date, path = 'https://www.web.statistik.zh.ch/zhwebcomponent/map/') {
    return await new Promise(function(resolve) {
      const actualDate = date ? new Date(date).getTime() : new Date().getTime()
      if (cachedMaps[key + date]) {
        resolve(zhutils.copyObj(cachedMaps[key + date]))
      } else {
        zhutils.getMapKeys(path).then(keys => {
          const k = keys.find(o => o.key === key)
          if (!k) {
            throw new Error(`map key ${key} not found in: [${ keys.map(o => o.key).join(', ') }]`)
          }
          const maps = k.maps
          const actualMap = maps.find(o => {
            return new Date(o.date_from).getTime() < actualDate && actualDate < new Date(o.date_to).getTime()
          })
          $axios.get(path + actualMap.url).then(response => {
            cachedMaps[key + date] = response.data
            resolve(zhutils.copyObj(cachedMaps[key + date]))
          })
        })
      }
    })
  },
  download (domObj, filename = 'svg', type = 'svg', callback) {
    setTimeout(() => {
      const rect = domObj.getBoundingClientRect()
      domObj.setAttribute('width', parseInt(rect.width))
      domObj.setAttribute('height', parseInt(rect.height))
      setTimeout(() => {
        switch (type) {
          case 'svg':
            toSvg(domObj)
              .then(dataUrl => {
                domObj.removeAttribute('width')
                domObj.removeAttribute('height')
                zhutils.triggerDownload(dataUrl, filename, 'svg')
                if (callback && typeof callback === 'function') {
                  callback()
                }
              })
            break
          case 'png':
            toPng(domObj)
              .then(dataUrl => {
                domObj.removeAttribute('width')
                domObj.removeAttribute('height')
                zhutils.triggerDownload(dataUrl, filename, 'png')
                if (callback && typeof callback === 'function') {
                  callback()
                }
              })
            break
          case 'jpeg':
            domObj.style.backgroundColor = '#FFF'
            toJpeg(domObj)
              .then(dataUrl => {
                domObj.removeAttribute('width')
                domObj.removeAttribute('height')
                domObj.style.backgroundColor = null
                zhutils.triggerDownload(dataUrl, filename, 'jpg')
                if (callback && typeof callback === 'function') {
                  callback()
                }
              })
            break
              
          default:
            break
        }
      }, 0)
    }, 0)
  },
  triggerDownload (imgURL, filename, type) {
    const link = document.createElement('a')
    link.setAttribute('href', imgURL)
    link.setAttribute('download', filename + '.' + type)
    link.style.visibility = 'hidden'
    document.body.appendChild(link)
    link.click()
    document.body.removeChild(link)
  },
  form (key = 'rect', s = 12, x = s / 2, y = s / 2) {
    switch (key) {
      case 'circle':
        s /= 2
        return `M${x-s},${y}a${s},${s} 0 1,0 ${Number(s*2).toFixed(3)},0a${s},${s} 0 1,0 -${Number(s*2).toFixed(3)},0`
      case 'diamond':
        s /= 2
        return `M${x} ${y+s},${x+s} ${y},${x} ${y-s},${x-s} ${y}Z`
      case 'rect':
        x -= s / 2
        y -= s / 2
        return `M${x} ${y},${x+s} ${y},${x+s} ${y+s},${x} ${y+s}Z`
      case 'triangle':
        x -= s / 2
        y -= s / 2
        return `M${Number(x+s/2).toFixed(3)} ${y},${x+s} ${y+s},${x} ${y+s}Z`
      case 'triangle-opposite':
        x -= s / 2
        y -= s / 2
        return `M${Number(x+s/2).toFixed(3)} ${y+s},${x+s} ${y},${x} ${y}Z`
      default:
        console.error(`Wrong form type: '${key}', use one of these: ['circle', 'diamond', 'rect', 'triangle', 'triangle-opposite']`)
        break
    }
  }
}
